@@ -267,7 +267,7 @@ class MyCnfRender(object):
         self.innodb_autoextend_increment = 64
         self.innodb_open_files = 100000
 
-        #### mgr
+        # mgr
         self.is_mgr = False
         self.group_replication_single_primary_mode = "ON"
         self.group_replication_group_name = "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"
@@ -315,7 +315,7 @@ class MyCnfRender(object):
             self.innodb_write_io_threads = 8
             self.innodb_purge_threads = 8
             self.innodb_page_cleaners = 8
-    
+
     def _config_mems(self):
         """
         根据 mem 核心数量重新配置配置文件中相关的参数
@@ -325,7 +325,7 @@ class MyCnfRender(object):
 
         chunk = 1
         if self.max_mem <= 512:
-            
+
             # 如果给定的内存小于 512M 那直接只分配一个 chunk 就行
             chunk = 1
         elif self.max_mem <= 1024:
@@ -335,34 +335,34 @@ class MyCnfRender(object):
         elif self.max_mem <= 4096:
 
             # 给定的内存大于 1G 但是小于 4G 分配 50%
-            chunk = int( (self.max_mem // 128) * 0.5 )
+            chunk = int((self.max_mem // 128) * 0.5)
         elif self.max_mem <= 1024 * 8:
 
             # 如果给定的内存在区间 （4G,8G] 内就分配 55%
-            chunk = int( (self.max_mem // 128) * 0.55 )
+            chunk = int((self.max_mem // 128) * 0.55)
         elif self.max_mem <= 1024 * 16:
 
             # 如果给定的内存在区间 （8G,16G] 内就分配 60%
-            chunk = int( (self.max_mem // 128) * 0.60 )
+            chunk = int((self.max_mem // 128) * 0.60)
         elif self.max_mem <= 1024 * 32:
 
             # 如果给定的内存在区间 （16G,32G] 内就分配 65%
-            chunk = int( (self.max_mem // 128) * 0.65 )
+            chunk = int((self.max_mem // 128) * 0.65)
         elif self.max_mem <= 1024 * 64:
 
             # 如果给定的内存在区间 （32G,64G] 内就分配 70%
-            chunk = int( (self.max_mem // 128) * 0.70 )
+            chunk = int((self.max_mem // 128) * 0.70)
         elif self.max_mem <= 1024 * 128:
 
             # 如果给定的内存在区间 （64G,128G] 内就分配 75%
-            chunk = int( (self.max_mem // 128) * 0.75 )
+            chunk = int((self.max_mem // 128) * 0.75)
         elif self.max_mem <= 1024 * 256:
 
             # 如果给定的内存在区间 （128G,256G] 内就分配 80%
-            chunk = int( (self.max_mem // 128) * 0.80 )
+            chunk = int((self.max_mem // 128) * 0.80)
         else:
             # 如果给定的内存在区间 （256G,++G] 内就分配 85%
-            chunk = int( (self.max_mem // 128) * 0.85 )
+            chunk = int((self.max_mem // 128) * 0.85)
 
         # 根据 chunk 的数量分配 innodb_buffer_pool_size
         if chunk % 4 == 0:
@@ -372,25 +372,26 @@ class MyCnfRender(object):
         else:
             # 不可以被 4 整除就用 M 做单位
             self.innodb_buffer_pool_size = f"{int(chunk * 128)}M"
-        logger.debug(f"change innodb_buffer_pool_size to {self.innodb_buffer_pool_size}")
-        
+        logger.debug(
+            f"change innodb_buffer_pool_size to {self.innodb_buffer_pool_size}")
 
         # 根据 chunk 的数量确定 innodb-buffer-pool-instances 的数量
         instances = chunk // 8 if chunk >= 8 else 1  # 至少每两 G 一个 instance
-        
+
         # 最多也就分配 32 个实例
         if instances >= 32:
             instances = 32
         self.innodb_buffer_pool_instances = instances
-        logger.debug(f"change innodb_buffer_pool_instances to {self.innodb_buffer_pool_instances}")
+        logger.debug(
+            f"change innodb_buffer_pool_instances to {self.innodb_buffer_pool_instances}")
 
         # 只有在 innodb_buffer_pool_size > 2G 的情况下才有必要开启 performance_schema
-        self.performance_schema  = 'ON' if chunk >=16 else 'OFF'
+        self.performance_schema = 'ON' if chunk >= 16 else 'OFF'
         logger.debug(f"change performance_schema to {self.performance_schema}")
 
         # dbm-agent 默认会分配 8 个 redo 文件
         if chunk <= 8:
-            
+
             # buffer_pool <= 1G
             self.innodb_log_file_size = '64M'
             self.innodb_log_buffer_size = '64M'
@@ -413,8 +414,8 @@ class MyCnfRender(object):
 
             # 8G < buffer_pool
             self.innodb_log_file_size = '256M'
-            self.innodb_log_buffer_size = '2G'        
-        
+            self.innodb_log_buffer_size = '2G'
+
     def _config_disk(self):
         """
         磁盘目前还不做配置
@@ -429,8 +430,8 @@ class MyCnfRender(object):
         self._config_mems()
         self._config_disk()
 
-    def enable_mgr(self,local_address:str= "127.0.0.1:33061",
-                  group_seeds:str="127.0.0.1:33061,127.0.0.1:33062,127.0.0.1:33063"):
+    def enable_mgr(self, local_address: str = "127.0.0.1:33061",
+                   group_seeds: str = "127.0.0.1:33061,127.0.0.1:33062,127.0.0.1:33063"):
         """
         启用 MGR
         """
@@ -502,7 +503,7 @@ class MyCnfInitRender(MyCnfRender):
     专门为 init 阶段生成配置文件所用的 render
     """
     logger = logger.getChild('MyCnfInitRender')
-    
+
     def render_template_file(self):
         """
         渲染
@@ -520,7 +521,7 @@ class MyCnfInitRender(MyCnfRender):
         self.set_output(init_cnf)
         self.set_template(template)
         self.render()
-    
+
         logger.info("render template file complete")
 
 
@@ -529,7 +530,7 @@ class MyCnfMSRender(MyCnfRender):
     专门为单机、主从 环境生成配置文件所用的 render
     """
     logger = logger.getChild('MyCnfMSRender')
-    
+
     def render_template_file(self):
         """
         渲染适合单机、主从的生产环境配置文件
@@ -543,14 +544,14 @@ class MyCnfMSRender(MyCnfRender):
         # 要使用的模板
         pattern = r"8.0.\d\d"
         version_number = re.search(pattern, self.basedir).group(0)
-        
+
         template = f'mysql-{version_number}.cnf.jinja'
         logger.info(f"using template '{template}' ")
 
         self.set_output(out_cnf_file)
         self.set_template(template)
         self.render()
-    
+
         logger.info("render template file complete")
 
 
@@ -562,25 +563,25 @@ class MyCnfMGRRender(MyCnfRender):
 
     def __init__(self, basedir="/usr/local/mysql-8.0.17-linux-glibc2.12-x86_64/",
                  datadir="/database/mysql/data/3306/", binlogdir="/binlog/mysql/binlog/3306/",
-                 max_mem=128, cores=1, port=3306, user="mysql3306",local_address:str= None,
-                  group_seeds:str=None
+                 max_mem=128, cores=1, port=3306, user="mysql3306", local_address: str = None,
+                 group_seeds: str = None
                  ):
         """
 
         """
         MyCnfRender.__init__(self, basedir=basedir, datadir=datadir, binlogdir=binlogdir, max_mem=max_mem,
                              cores=cores, port=port, user=user)
-        
+
         # group_seeds 和 local_address 都不能是 None
         if local_address is None or group_seeds is None:
-            raise ValueError("args('local_address','group_seeds') in MyCnfMGRRender.__init__ must be not None")
+            raise ValueError(
+                "args('local_address','group_seeds') in MyCnfMGRRender.__init__ must be not None")
 
         self.group_replication_local_address = local_address
         self.group_replication_group_seeds = group_seeds
         self.group_replication_consistency = "BEFORE_ON_PRIMARY_FAILOVER"
         self.is_mgr = True
 
-
     def render_template_file(self):
         """
         渲染适用于 MGR 的生产环境配置文件
@@ -594,17 +595,17 @@ class MyCnfMGRRender(MyCnfRender):
         # 要使用的模板
         pattern = r"8.0.\d\d"
         version_number = re.search(pattern, self.basedir).group(0)
-        
+
         template = f'mysql-{version_number}.cnf.jinja'
         logger.info(f"using template '{template}' ")
 
         self.set_output(out_cnf_file)
         self.set_template(template)
         self.render()
-    
+
         logger.info("render template file complete")
 
-        
+
 class MySQLSystemdRender(object):
     """
     渲染 systemd 会用到的 mysqld-{self.port}.service 配置文件
@@ -629,7 +630,7 @@ class MySQLSystemdRender(object):
         self.tmpl = Environment(loader=FileSystemLoader(
             searchpath=template_dir)).get_template(template_file)
 
-    def set_output(self,output_file="/usr/lib/systemd/system/mysqld-3306.service"):
+    def set_output(self, output_file="/usr/lib/systemd/system/mysqld-3306.service"):
         """
         设置输出的目标文件
         """
@@ -646,15 +647,14 @@ class MySQLSystemdRender(object):
         logger.debug("start render mysql systemd config file")
 
         if not hasattr(self, 'tmpl'):
-            
+
             # 如果还没有 template 对象就运行它
             self.set_template()
 
         if not hasattr(self, '_output'):
-            
-            # 如果还没有 _output 
+
+            # 如果还没有 _output
             self.set_output()
-        
 
         # 配置全局字典
         self.tmpl.globals = self.__dict__
@@ -939,7 +939,7 @@ class MySQLInstallerMixin(object):
 
         # 创建渲染对象
         render = MyCnfInitRender(basedir=self.basedir, datadir=self.datadir, binlogdir=self.binlogdir,
-                             max_mem=self.max_mem, cores=self.cores, port=self.port, user=self.user)
+                                 max_mem=self.max_mem, cores=self.cores, port=self.port, user=self.user)
         render.render_template_file()
 
     def _render_production_cnf(self):
@@ -949,7 +949,7 @@ class MySQLInstallerMixin(object):
         logger = self.logger.getChild('_render_production_cnf')
 
         render = MyCnfMSRender(basedir=self.basedir, datadir=self.datadir, binlogdir=self.binlogdir,
-                             max_mem=self.max_mem, cores=self.cores, port=self.port, user=self.user)
+                               max_mem=self.max_mem, cores=self.cores, port=self.port, user=self.user)
 
         # 开始渲染
         render.render_template_file()
@@ -964,8 +964,8 @@ class MySQLInstallerMixin(object):
 
         # 创建配置文件生成器
         render = MyCnfMGRRender(basedir=self.basedir, datadir=self.datadir, binlogdir=self.binlogdir,
-                             max_mem=self.max_mem, cores=self.cores, port=self.port, user=self.user,
-                             local_address=self.local_address,group_seeds=self.group_seeds)
+                                max_mem=self.max_mem, cores=self.cores, port=self.port, user=self.user,
+                                local_address=self.local_address, group_seeds=self.group_seeds)
         render.render_template_file()
 
     def _init_database(self):
@@ -989,7 +989,7 @@ class MySQLInstallerMixin(object):
                     self._render_production_cnf()
             else:
                 self._render_production_cnf()
-        
+
         # 强行渲染 MGR 的配置文件
         if not checkings.is_file_exists(f"/etc/my-{self.port}.cnf"):
             if hasattr(self, 'is_mgr'):
@@ -1002,7 +1002,8 @@ class MySQLInstallerMixin(object):
             args = [f'{self.basedir}/bin/mysqld', f'--defaults-file=/tmp/mysql-init.cnf',
                     '--initialize-insecure', f'--user=mysql{self.port}', f'--init-file={init_file}']
             logger.info(args)
-            subprocess.run(args, stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+            subprocess.run(args, stdout=subprocess.PIPE,
+                           stderr=subprocess.PIPE)
 
         # 删除初始化文件
         logger.debug("remove /tmp/mysql-init.cnf")
@@ -1018,10 +1019,11 @@ class MySQLInstallerMixin(object):
 
         render = MySQLSystemdRender(
             basedir=self.basedir, port=self.port, user=self.user)
-        
+
         # 配置
         render.set_template("mysqld.service.jinja")
-        render.set_output(f"/usr/lib/systemd/system/mysqld-{self.port}.service")
+        render.set_output(
+            f"/usr/lib/systemd/system/mysqld-{self.port}.service")
 
         render.render()
         logger.info("mysql systemd config complete")
@@ -1032,9 +1034,10 @@ class MySQLInstallerMixin(object):
         """
         logger = self.logger.getChild("_enable_mysql")
         with common.sudo(f"enable systemd mysqld-{self.port}"):
-            subprocess.run(['systemctl daemon-reload'], shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+            subprocess.run(['systemctl daemon-reload'], shell=True,
+                           stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             subprocess.run(
-                [f'systemctl enable mysqld-{self.port}'], shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+                [f'systemctl enable mysqld-{self.port}'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         logger.info("config mysql auto start on boot complete")
 
     def _start_mysql(self):
@@ -1044,7 +1047,7 @@ class MySQLInstallerMixin(object):
         logger = self.logger.getChild("_start_mysql")
         with common.sudo(f"enable systemd mysqld-{self.port}"):
             subprocess.run(
-                [f'systemctl start mysqld-{self.port}'], shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
+                [f'systemctl start mysqld-{self.port}'], shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
         logger.debug("wait mysql start")
 
@@ -1083,7 +1086,7 @@ class MySQLInstallerMixin(object):
         """
         logger = self.logger.getChild('_export_so')
         export_file = os.path.join(
-            '/etc/ld.so.conf.d', f'{self._get_mysql_version()}')
+            '/etc/ld.so.conf.d', f'{self._get_mysql_version()}.conf')
         if not checkings.is_file_exists(export_file):
             with common.sudo("export so file"):
                 with open(export_file, 'w') as sofile:
@@ -1149,7 +1152,7 @@ class MySQLInstallerMixin(object):
 
         logger.info("install mysql single instance complete")
 
-    
+
 class MySQLUninstallerMixin(object):
     """
     实现 MySQL 卸载相关的逻辑
@@ -1163,7 +1166,7 @@ class MySQLUninstallerMixin(object):
 
         logger.debug(f"doing checks for uninstall mysql instance {self.port}")
         # 检查项1、端口不能在使用中
-        if checkings.is_port_in_use(ip="127.0.0.1",port=self.port):
+        if checkings.is_port_in_use(ip="127.0.0.1", port=self.port):
 
             # 卸载前要确保端口没有被占用
             logger.error(f"port '{self.port}' is in use")
@@ -1174,22 +1177,24 @@ class MySQLUninstallerMixin(object):
             logger.error(f"pid file  '/tmp/mysql-{self.port}.pid' exists")
             raise errors.FileAlreadyExistsError(f"/tmp/mysql-{self.port}.pid")
 
-
     def uninstall(self):
         """
         """
         logger = self.logger.getChild('uninstall')
-        
+
         try:
             self._basic_checks()
         except errors.PortIsInUseError as err:
-            logger.warning(f"mysql is runing,want stop it? 'systemctl stop mysqld-{self.port}' ")
+            logger.warning(
+                f"mysql is runing,want stop it? 'systemctl stop mysqld-{self.port}' ")
             return
         except errors.FileAlreadyExistsError as err:
-            logger.warning(f"mysql is runing,want stop it? 'systemctl stop mysqld-{self.port}' ")
+            logger.warning(
+                f"mysql is runing,want stop it? 'systemctl stop mysqld-{self.port}' ")
             return
         except Exception as err:
-            logger.error(f"undefined exception occure,can not remove this mysql instance {err}")
+            logger.error(
+                f"undefined exception occure,can not remove this mysql instance {err}")
             return
 
         # 能执行到这里说明数据库实例已经停止运行
@@ -1199,7 +1204,7 @@ class MySQLUninstallerMixin(object):
             if checkings.is_user_exists(self.user):
                 common.delete_user(self.user)
                 logger.info(f"user '{self.user}' deleted")
-            
+
             with common.sudo(f"drop instance {self.port}"):
                 # 删除数据目录
                 if checkings.is_directory_exists(self.datadir):
@@ -1217,8 +1222,9 @@ class MySQLUninstallerMixin(object):
                     logger.info(f"backup direcotry '{self.backupdir}' deleted")
 
                 # 去除开机自启动
-                subprocess.run(f"systemctl disable mysqld-{self.port}", shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
-                
+                subprocess.run(
+                    f"systemctl disable mysqld-{self.port}", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
                 # 删除配置文件
                 if checkings.is_file_exists(self.cnf_file):
                     os.remove(self.cnf_file)
@@ -1247,7 +1253,7 @@ class MySQLClonerMixin(object):
         """
         logger = self.logger.getChild("_local_clone_checks")
         # 检查 MySQL 数据库是否在运行
-        if not checkings.is_port_in_use(ip='127.0.0.1',port=self.port):
+        if not checkings.is_port_in_use(ip='127.0.0.1', port=self.port):
 
             # 如果端口还没有在使用，说明本地克隆的条件不成立
             logger.error(f"mysqld-{self.port} is not runing")
@@ -1256,13 +1262,15 @@ class MySQLClonerMixin(object):
         # 检查 MySQL 是否有权限连接到要克隆的实例
         cnx = None
         try:
-            cnx = connector.connect(host=self.host, port=self.port, user=self.clone_user, password=self.clone_password)
+            cnx = connector.connect(
+                host=self.host, port=self.port, user=self.clone_user, password=self.clone_password)
             cursor = cnx.cursor(dictionary=True)
             cursor.execute("select 1 as ok")
             data = cursor.fetchone()
-            
+
         except Exception as err:
-            logger.error(f"'cloneuser' conect to {self.host} {self.port} faile innner error {err}")
+            logger.error(
+                f"'cloneuser' conect to {self.host} {self.port} faile innner error {err}")
             raise err
 
         finally:
@@ -1278,7 +1286,8 @@ class MySQLClonerMixin(object):
         logger = self.logger.getChild('_remote_clone_checks')
         try:
             logger.debug("")
-            cnx = connector.connect(host=self.host, port=self.port, user=self.clone_user, password=self.clone_password)
+            cnx = connector.connect(
+                host=self.host, port=self.port, user=self.clone_user, password=self.clone_password)
             cursor = cnx.cursor(dictionary=True)
             cursor.execute("select 1 as ok;")
             data = cursor.fetchone()
@@ -1323,7 +1332,7 @@ class MySQLShellInstallerMixin(object):
         MySQL-shell 是否已经完成安装
         """
         return checkings.is_directory_exists(self.mysql_shell_basedir)
-        
+
     def _extract_install_pkg(self):
         """
         解压安装包到 /usr/local/
@@ -1332,24 +1341,25 @@ class MySQLShellInstallerMixin(object):
         if self._is_shell_installed == True:
             logger.info("mysql-shell has been installed")
             return
-            
+
         # 能执行到这里说明 mysql-shell 还没有被安装
 
         # 解压安装包
         with common.sudo(f"extract mysql-shell install package to {self.dbma.mysql_install_dir}"):
-            shutil.unpack_archive(self.mysql_shell_pkg_full_path, self.dbma.mysql_install_dir)
+            shutil.unpack_archive(
+                self.mysql_shell_pkg_full_path, self.dbma.mysql_install_dir)
             common.recursive_change_owner(
                 path=self.mysql_shell_basedir, user='root', group='mysql')
 
         logger.info("extract mysql-shell complete")
-    
+
     def _export_path(self):
         """
         导出 PATH 环境变量
         """
         logger = self.logger.getChild("_export_path")
 
-        # 计算出 export 
+        # 计算出 export
         shell_bin_path = os.path.join(self.mysql_shell_basedir, 'bin')
         line = f"export PATH={shell_bin_path}:$PATH"
 
@@ -1358,7 +1368,7 @@ class MySQLShellInstallerMixin(object):
             with open('/etc/profile') as f_profile:
                 logger.info("mysql-shell path not exported")
                 is_exported = line in f_profile
-        
+
         # 如果没有
         if is_exported == False:
             with common.sudo("export mysql-shell to path"):
@@ -1368,9 +1378,9 @@ class MySQLShellInstallerMixin(object):
                     f_profile.write('\n')
 
                 logger.info("mysql-shell path exported")
-        
+
         logger.info("export mysql-shell path complete")
-    
+
     def install(self):
         """
         """
@@ -1394,7 +1404,7 @@ class MySQLShellInstallerMixin(object):
 
         logger.info("install mysql-shell complete")
 
-    
+
 class MySQLInstaller(threading.Thread, MySQLInstallerMixin):
     """
     实现单机环境的安装
@@ -1450,7 +1460,7 @@ class MySQLUninstaller(threading.Thread, MySQLUninstallerMixin):
         """
         logger = self.logger.getChild('MysqlUninstaller')
 
-        threading.Thread.__init__(self,name='um',daemon=True)
+        threading.Thread.__init__(self, name='um', daemon=True)
         self.port = port
         self.user = f"mysql{self.port}"
         self.datadir = f"/database/mysql/data/{self.port}"
@@ -1472,7 +1482,7 @@ class MySQLUninstaller(threading.Thread, MySQLUninstallerMixin):
         self.uninstall()
 
 
-class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
+class MySQLCloner(threading.Thread, MySQLClonerMixin, MySQLInstallerMixin):
     """
     实现对 MYSQL 实例的克隆，同时支持本地克隆和远程克隆两种方式
 
@@ -1487,13 +1497,14 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
     logger = logger.getChild('MySQLCloner')
 
     def __init__(self, pkg="mysql-8.0.18-linux-glibc2.12-x86_64.tar.xz",
-                 host="127.0.0.1",port=3306,clone_user="cloneuser",clone_password="dbma@0352",max_mem=128, cores=1,name='cm',daemon=True):
+                 host="127.0.0.1", port=3306, clone_user="cloneuser", clone_password="dbma@0352", max_mem=128, cores=1, name='cm', daemon=True):
         """
 
         可以根据 host 的地址来决定是本地克隆还是远程克隆
         """
         threading.Thread.__init__(self, name=name, daemon=daemon)
-        logger.debug(f"cloner instance init args pkg={pkg} host={host} port={port} user={clone_user} password={clone_password} max_mem={max_mem} cores={cores} ")
+        logger.debug(
+            f"cloner instance init args pkg={pkg} host={host} port={port} user={clone_user} password={clone_password} max_mem={max_mem} cores={cores} ")
 
         # 取得 dbma 的配置文件对象
         self.dbma = cnf
@@ -1513,14 +1524,16 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
     def _is_remote_mgr_enabled(self):
         """
         要克隆远程主机是否开启 MGR
-        
+
         通过检查是否存在 group_replication_group_name 变量来判断是否有开启 MGR
         """
         cnx = None
         try:
-            cnx = connector.connect(host=self.host, port=self.port, user=self.clone_user, password=self.clone_password)
+            cnx = connector.connect(
+                host=self.host, port=self.port, user=self.clone_user, password=self.clone_password)
             cursor = cnx.cursor(dictionary=True)
-            cursor.execute("show global variables like 'group_replication_group_name'")
+            cursor.execute(
+                "show global variables like 'group_replication_group_name'")
             data = cursor.fetchone()
             is_mgr_enabled = True if data is not None else False
         except Exception as err:
@@ -1532,32 +1545,36 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
         self.is_mgr = is_mgr_enabled
 
         if type(self) == MySQLBuildSlave:
-            
+
             # 如果只是创建 slave 强行把 mgr 切为 false
             self.is_mgr = False
-        
+
     def local_clone(self):
         """
         克隆当前主机上的实例
         """
         logger = self.logger.getChild("local_clone")
         logger.info(f"start clone mysqld-{self.port}")
-        logger.info(f"save backup files to '/backup/mysql/backup/{self.port}/{self.now}'")
+        logger.info(
+            f"save backup files to '/backup/mysql/backup/{self.port}/{self.now}'")
         try:
             # 如果在检查的时候就遇到了异常那就直接报
             self._local_clone_checks()
         except Exception as err:
-            logger.error(f"can't clone current instance mysqld{self.port} inner error {err}")
+            logger.error(
+                f"can't clone current instance mysqld{self.port} inner error {err}")
             return
 
         cnx = None
-        #更新一下备份目录的权限
+        # 更新一下备份目录的权限
 
         with common.sudo("change backup directory's owner"):
-            shutil.chown(f"/backup/mysql/backup/{self.port}/", f'mysql{self.port}')
-        
+            shutil.chown(
+                f"/backup/mysql/backup/{self.port}/", f'mysql{self.port}')
+
         try:
-            cnx = connector.connect(host='127.0.0.1', port=self.port, user=self.clone_user, password=self.clone_password)
+            cnx = connector.connect(
+                host='127.0.0.1', port=self.port, user=self.clone_user, password=self.clone_password)
             cursor = cnx.cursor(dictionary=True)
             local_clone_sql = f"CLONE LOCAL DATA DIRECTORY = '/backup/mysql/backup/{self.port}/{self.now}' "
             cursor.execute(local_clone_sql)
@@ -1579,7 +1596,7 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
         try:
             self._remote_clone_checks()
         except Exception as err:
-            
+
             # 检查时遇到错误，clone直接失败
             logger.error(f"remote clone check fail {err}")
             return
@@ -1594,10 +1611,11 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
             logger.error(f"{err}")
             return
 
-        # 验证远程实例是不是 MGR 
+        # 验证远程实例是不是 MGR
         self._is_remote_mgr_enabled()
         if self.is_mgr == True:
-            logger.warning("remonte instance is an mgr node, dbm well render mgr options")
+            logger.warning(
+                "remonte instance is an mgr node, dbm well render mgr options")
 
         # 安装一个本地的 MYSQL 实例
         self.install()
@@ -1605,9 +1623,10 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
         # 第二步：连接上本地实例，然后从远程克隆
         cnx = None
         try:
-            
+
             # 连接到本地实例并设置它的 donor list
-            cnx = connector.connect(host="127.0.0.1", port=self.port, user=self.clone_user, password=self.clone_password)
+            cnx = connector.connect(
+                host="127.0.0.1", port=self.port, user=self.clone_user, password=self.clone_password)
             donor_sql = f"set @@global.clone_valid_donor_list='{self.host}:{self.port}';"
             cursor = cnx.cursor(dictionary=True)
             logger.info(f"prepare execute '{donor_sql}' ")
@@ -1617,18 +1636,19 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
             clone_sql = f"clone instance from {self.clone_user}@'{self.host}':{self.port} identified by '{self.clone_password}';"
             logger.info(f"prepare execute '{clone_sql}' ")
             cursor.execute(clone_sql)
-            
+
         except Exception as err:
             logger.info(f"got some error during remonte clone {err}")
-            return 
+            return
         finally:
             if hasattr(cnx, 'close'):
                 cnx.close()
 
         # 可以执行到这里说明 remote_clone 成功了
-        logger.info("remote clone complete (mysql will auto restart,please wait)")
-        self.is_successful_complete = True 
-        
+        logger.info(
+            "remote clone complete (mysql will auto restart,please wait)")
+        self.is_successful_complete = True
+
     def run(self):
         """
         """
@@ -1636,7 +1656,7 @@ class MySQLCloner(threading.Thread, MySQLClonerMixin,MySQLInstallerMixin):
             self.local_clone()
         else:
             self.remote_clone()
-    
+
 
 class MySQLBuildSlave(MySQLCloner):
     """
@@ -1650,18 +1670,19 @@ class MySQLBuildSlave(MySQLCloner):
 
     def __init__(self, pkg="mysql-8.0.18-linux-glibc2.12-x86_64.tar.xz",
                  host="127.0.0.1", port=3306, clone_user="cloneuser", clone_password="dbma@0352",
-                 replication_user="repluser",replication_password="dbma@0352",
+                 replication_user="repluser", replication_password="dbma@0352",
                  max_mem=128, cores=1, name='cm', daemon=True):
-        
+
         MySQLCloner.__init__(self, pkg=pkg, host=host, port=port, clone_user=clone_user, clone_password=clone_password,
                              max_mem=max_mem, cores=cores, name=name, daemon=daemon)
-        
+
         # 添加 build slave 相关的参数
         logger = self.logger.getChild("__init__")
         self.replication_user = replication_user
         self.replication_password = replication_password
 
-        logger.debug(f"init MySQLBuildSlave instance replication_user = '{self.replication_user}'  replication_password='{self.replication_password}' ")
+        logger.debug(
+            f"init MySQLBuildSlave instance replication_user = '{self.replication_user}'  replication_password='{self.replication_password}' ")
 
     def build_slave(self):
         """
@@ -1673,7 +1694,7 @@ class MySQLBuildSlave(MySQLCloner):
 
         # 第二步、检查远程克隆是否成功完成
         if not self.is_successful_complete == True:
-            
+
             # 远程克隆没有成功完成
             logger.error("remote clone fail can't rebuid slave")
             return
@@ -1693,19 +1714,20 @@ class MySQLBuildSlave(MySQLCloner):
             logger.warning(f"wait mysqld-{self.port} avariable")
             time.sleep(1)
         else:
-            logger.error(f"mysqld-{self.port} not avariable concat your dba for help")
+            logger.error(
+                f"mysqld-{self.port} not avariable concat your dba for help")
 
-        
         # 第三步、执行 change master to & start slave
 
         cnx = None
         try:
-            logger.debug(f"connect to mysqld-{self.port} with user={self.replication_user} password={self.replication_password}")
+            logger.debug(
+                f"connect to mysqld-{self.port} with user={self.replication_user} password={self.replication_password}")
             cnx = connector.connect(host="127.0.0.1", port=self.port,
                                     user=self.replication_user, password=self.replication_password)
             cursor = cnx.cursor()
             change_sql = f"change master to master_host='{self.host}',master_port={self.port},master_user='{self.replication_user}',master_password='{self.replication_password}',master_ssl = 1,master_auto_position=1;"
-            
+
             # 准备执行 change master to
             logger.info(f"prepare execute '{change_sql}'")
             cursor.execute(change_sql)
@@ -1720,7 +1742,7 @@ class MySQLBuildSlave(MySQLCloner):
 
         except Exception as err:
             logger.error(f"config replication slave got error {err}")
-        
+
         finally:
             if hasattr(cnx, 'close'):
                 cnx.close()
@@ -1731,8 +1753,8 @@ class MySQLBuildSlave(MySQLCloner):
         """
         """
         self.build_slave()
-    
-        
+
+
 class MySQLBuildMGR(MySQLCloner):
     """
     实现 MySQL-MGR 的自动化安装与配置
@@ -1741,7 +1763,7 @@ class MySQLBuildMGR(MySQLCloner):
 
     def __init__(self, pkg="mysql-8.0.18-linux-glibc2.12-x86_64.tar.xz",
                  port=3306, members="127.0.0.1,127.0.0.2,127.0.0.3", max_mem=128, cores=1,
-                 clone_password='dbma@0352',repl_password='dbma@0352',name='imgr', daemon=True):
+                 clone_password='dbma@0352', repl_password='dbma@0352', name='imgr', daemon=True):
         """
         MGR 环境的安装与配置
 
@@ -1770,14 +1792,17 @@ class MySQLBuildMGR(MySQLCloner):
         logger.info(f"using {host}:{port} as primary node")
 
         MySQLCloner.__init__(self, pkg=pkg, host=host, port=port, clone_user=clone_user, clone_password=clone_password,
-                             max_mem = max_mem, cores = cores, name = name, daemon = daemon)
-        
-        logger.debug(f"create {self.__class__.__name__} instance using pkg='{pkg}' port='{port}' members='{members}' max_mem='{max_mem}' cores='{cores}'")
+                             max_mem=max_mem, cores=cores, name=name, daemon=daemon)
 
-        logger.warning(f"clone user info user='{clone_user}' password='{clone_password}' ")
+        logger.debug(
+            f"create {self.__class__.__name__} instance using pkg='{pkg}' port='{port}' members='{members}' max_mem='{max_mem}' cores='{cores}'")
+
+        logger.warning(
+            f"clone user info user='{clone_user}' password='{clone_password}' ")
 
         # 配置 group_replication_group_seeds
-        self.group_seeds = f":{port*10 + 1},".join(members.split(',')) + f":{port*10 + 1}"
+        self.group_seeds = f":{port*10 + 1},".join(
+            members.split(',')) + f":{port*10 + 1}"
         logger.info(f"group_replication_group_seeds = {self.group_seeds}")
 
         # 配置 group_replication_local_address
@@ -1785,7 +1810,7 @@ class MySQLBuildMGR(MySQLCloner):
         member_ip_set = set()
         for ip in members.split(','):
             member_ip_set.add(ip)
-        
+
         # 记录下所有成员的 IP 地址列表
         self.member_ip_list = [ip for ip in member_ip_set]
 
@@ -1815,14 +1840,14 @@ class MySQLBuildMGR(MySQLCloner):
             MySQLInstallerMixin._basic_checks(self)
         except Exception as err:
             raise err
-            
+
         # 如果可以执行到这里说明基本的安装检查项通过了
         # 检查 mgr 同步用的端口是否被占用
         if checkings.is_port_in_use(ip="127.0.0.1", port=self.port * 10 + 1):
             logger.error(f"MGR port {self.port * 10 + 1} is in use ")
-            raise errors.PortIsInUseError(f"MGR port {self.port * 10 + 1} is in use ")
+            raise errors.PortIsInUseError(
+                f"MGR port {self.port * 10 + 1} is in use ")
 
-        
         # 检查主机间的 DNS 是否
         for member_ip in self.member_ip_list:
 
@@ -1843,12 +1868,13 @@ class MySQLBuildMGR(MySQLCloner):
         with common.sudo("config dns for mgr"):
 
             for member_ip in self.member_ip_list:
-                
+
                 # 配置主机名
                 # dbm-127-0-0-1
                 hostname = "dbm-{}".format(member_ip.split('.', '-'))
-                subprocess.run(f"hostnamectl set-hostname {hostname} ", shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
-                
+                subprocess.run(
+                    f"hostnamectl set-hostname {hostname} ", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
                 # 配置 /etc/hosts
                 line = f"{member_ip}    {hostname}"
                 with open("/etc/hosts", 'a') as f_host:
@@ -1860,31 +1886,63 @@ class MySQLBuildMGR(MySQLCloner):
         """
         logger = self.logger.getChild("build_primary")
         try:
-            
+
             # 检查 是否满足 MGR 相关的配置
             # 如果是 DNS 的问题 _mgr_check 会自动配置
             self._mgr_checks()
         except Exception as err:
             logger.error("build MGR primary node faild")
             return
-        
+
         # 开始安装 primary 结点
         # 因为 self.is_mgr == True 所以 install 会自动配置 MGR 的内容
         self.install()
-        
-        # 启动 MGR 
+
+        # 安装完成之后要等待启动完成
+        wait_counter = 30
+        for _ in range(wait_counter):
+            if checkings.is_port_in_use(ip="127.0.0.1", port=self.port):
+
+                #
+                logger.info("mysql protocol avriable")
+                break
+
+            logger.info("wait 1 seconds for mysql protocol avriable")
+            time.sleep(1)
+
+        # 强制 sleep 7 秒
+        time.sleep(7)
+
+        # 启动 MGR
         cnx = None
         try:
 
             # 连接到已经安装好的 primary 结点
-            logger.info(f"connector to primary node(127.0.0.1:{self.port}) user='dbma' password='{cnf.init_pwd}' ")
+            logger.info(
+                f"connector to primary node(127.0.0.1:{self.port}) user='dbma' password='{cnf.init_pwd}' ")
 
-            cnx = connector.connect(host="127.0.0.1", port=self.port, user="dbma", password=cnf.init_pwd)
+            cnx = connector.connect(
+                host="127.0.0.1", port=self.port, user="dbma", password=cnf.init_pwd)
             cursor = cnx.cursor()
-            mgr_sql = "set @@global.group_replication_bootstrap_group=ON;start group_replication;set @@global.group_replication_bootstrap_group=OFF;"
+            # 直接一步到位有可能会创建 primary 失败(未知的原因)
+            #mgr_sql = "set @@global.group_replication_bootstrap_group=ON;start group_replication;set @@global.group_replication_bootstrap_group=OFF;"
             # 打印出 primary 结点上执行的语句
-            logger.info(mgr_sql)
-            cursor.execute(mgr_sql, multi=True)
+
+            # 第一步：打开开关
+            sql = "set @@global.group_replication_bootstrap_group=ON;"
+            logger.info(sql)
+            cursor.execute(sql)
+
+            # 第二步：启动 MGR
+            sql = "start group_replication;"
+            logger.info(sql)
+            cursor.execute(sql)
+
+            # 第三步：关闭开关
+            sql = "set @@global.group_replication_bootstrap_group=OFF;"
+            logger.info(sql)
+            cursor.execute(sql)
+
         except Exception as err:
 
             # 配置 primary 结点遇到了异常
@@ -1917,11 +1975,11 @@ class MySQLBuildMGR(MySQLCloner):
         self.remote_clone()
 
         if not self.is_successful_complete == True:
-            
+
             # 远程克隆没有成功完成
             logger.error("remote clone fail can't rebuid slave")
             return
-    
+
         logger.info('wait mysql restart complete ...')
         wait_counter = 90
         for _ in range(wait_counter):
@@ -1933,18 +1991,20 @@ class MySQLBuildMGR(MySQLCloner):
             logger.debug("wait 1 seconds for mysql protoco available")
             time.sleep(1)
         else:
-            
+
             # 没有被 break
-            logger.error("after remote clone mysql restart fail ,concat your dba for help")
+            logger.error(
+                "after remote clone mysql restart fail ,concat your dba for help")
             logger.error("build MGR seconder fail")
             return
-        
 
         # 配置 seconder 节点
         cnx = None
         try:
-            logger.debug(f"connect to 127.0.0.1:{self.port} user='repluser' password={self.repl_password} ")
-            cnx = connector.connect(host="127.0.0.1", port=self.port, user="repluser", password=self.repl_password)
+            logger.debug(
+                f"connect to 127.0.0.1:{self.port} user='repluser' password={self.repl_password} ")
+            cnx = connector.connect(
+                host="127.0.0.1", port=self.port, user="repluser", password=self.repl_password)
             cursor = cnx.cursor()
             change_sql = f"change master to master_user='repluser',master_password='{self.repl_password}' for channel 'group_replication_recovery';"
             logger.info(change_sql)
@@ -1961,7 +2021,7 @@ class MySQLBuildMGR(MySQLCloner):
                 cnx.close()
 
         if self.is_successful_complete == False:
-            
+
             # 说明 remote clone 没有成功
             logger.error("build MGR seconder node fail")
             return
@@ -1991,19 +2051,13 @@ class MySQLShellInstaller(threading.Thread, MySQLShellInstallerMixin):
 
         self.dbma = cnf
         self.pkg = pkg
-        self.mysql_shell_basedir = os.path.join(self.dbma.mysql_install_dir, self.pkg.replace('.tar.gz', '').replace('.tar.xz', ''))
-        self.mysql_shell_pkg_full_path = os.path.join(self.dbma.base_dir, 'pkg', self.pkg)
+        self.mysql_shell_basedir = os.path.join(
+            self.dbma.mysql_install_dir, self.pkg.replace('.tar.gz', '').replace('.tar.xz', ''))
+        self.mysql_shell_pkg_full_path = os.path.join(
+            self.dbma.base_dir, 'pkg', self.pkg)
         self.is_successful_complete = False
 
     def run(self):
         """
         """
         self.install()
-    
-
-    
-    
-        
-
-
-
